@@ -1,12 +1,8 @@
 import * as expect from 'expect';
 import { createExec, createExecTester } from './exec-helpers';
-import {
-  CMD_TS_NODE_WITH_PROJECT_FLAG,
-  contextTsNodeUnderTest,
-  TEST_DIR,
-} from './helpers';
-import { test as _test } from './testlib';
-const test = _test.context(contextTsNodeUnderTest);
+import { CMD_TS_NODE_WITH_PROJECT_FLAG, ctxTsNode, TEST_DIR } from './helpers';
+import { context } from './testlib';
+const test = context(ctxTsNode);
 
 const exec = createExecTester({
   cmd: CMD_TS_NODE_WITH_PROJECT_FLAG,
